
var pathRegexp = require('path-to-regexp');
var debug = require('debug')('express:router:layer');

function Layer(path, options, fn) {
  if (!(this instanceof Layer)) {
    return new Layer(path, options, fn);
  }

  debug('new %s', path);
  options = options || {};
  this.path = path;
  this.params = {};
  this.regexp = pathRegexp(path, this.keys = [], options);
  this.handle = fn;
}

/**
 * Check if this route matches `path`, if so
 * populate `.params`.
 *
 * @param {String} path
 * @return {Boolean}
 * @api private
 */

Layer.prototype.match = function(path){
  var keys = this.keys
    , params = this.params = {}
    , m = this.regexp.exec(path)
    , n = 0;
  var key;
  var val;

  if (!m) return false;

  for (var i = 1, len = m.length; i < len; ++i) {
    key = keys[i - 1];

    try {
      val = 'string' == typeof m[i]
        ? decodeURIComponent(m[i])
        : m[i];
    } catch(e) {
      var err = new Error("Failed to decode param '" + m[i] + "'");
      err.status = 400;
      throw err;
    }

    if (key) {
      params[key.name] = val;
    } else {
      params[n++] = val;
    }
  }

  return true;
};

module.exports = Layer;
